﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LifeLogApp.Services
{
    public class GeoService
    {
        // https://learn.microsoft.com/en-us/dotnet/maui/platform-integration/device/geolocation?view=net-maui-7.0&tabs=ios

        private Serilog.ILogger logger;

        private CancellationTokenSource cancelTokenSource;
        private bool isCheckingLocation;

        public GeoService(Serilog.ILogger logger)
        {
            this.logger = logger;
        }

        public async Task<Location> GetCurrentLocationAsync()
        {
            try
            {
                isCheckingLocation = true;

                GeolocationRequest request = new GeolocationRequest(GeolocationAccuracy.Medium, TimeSpan.FromSeconds(10));

                cancelTokenSource = new CancellationTokenSource();

                Location location = await Geolocation.Default.GetLocationAsync(request, cancelTokenSource.Token);

                if (location != null)
                {
                    logger.Debug("Latitude={Latitude}, Longitude={Longitude}, Speed={Speed}, Course={Course}, Accuracy={Accuracy}",
                        location.Latitude, location.Longitude, location.Speed, location.Course, location.Accuracy);
                }
                else
                {
                    logger.Error("No location");
                }

                return location;
            }
            // Catch one of the following exceptions:
            //   FeatureNotSupportedException
            //   FeatureNotEnabledException
            //   PermissionException
            catch (Exception ex)
            {
                logger.Error(ex, "No location");
            }
            finally
            {
                isCheckingLocation = false;
            }

            return null;
        }

        public void CancelCurrentLocationRequest()
        {
            if (isCheckingLocation && cancelTokenSource != null && cancelTokenSource.IsCancellationRequested == false)
                cancelTokenSource.Cancel();
        }

        public async Task<IEnumerable<Placemark>> GetReverseGeocodeDataAsync(double latitude, double longitude)
        {
            // https://learn.microsoft.com/en-us/dotnet/maui/platform-integration/device/geocoding?view=net-maui-7.0&tabs=ios

            var placemarks = await Geocoding.Default.GetPlacemarksAsync(latitude, longitude);

            return placemarks;
        }

        public string BuildFormattedAddress(Placemark placemark)
        {
            var street = placemark.Thoroughfare;
            var houseNo = placemark.FeatureName;
            if (string.IsNullOrWhiteSpace(houseNo)) { street = placemark.SubThoroughfare; }
            var postalCode = placemark.PostalCode;
            var city = placemark.Locality;
            if (string.IsNullOrWhiteSpace(city)) { city = placemark.SubLocality; }
            var formattedAddres = $"{street} {houseNo}, {postalCode}, {placemark.SubLocality}, {placemark.CountryCode}";

            return formattedAddres;
        }
    }
}
