﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using LifeLogApp.Classes;
using LifeLogApp.Interfaces;
using LifeLogApp.Model;
using LifeLogApp.Services;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace LifeLogApp.ViewModel
{
    // Binding: https://www.youtube.com/watch?v=5Qga2pniN78&t=2s

    public partial class MainViewModel : ObservableObject
    {
        Serilog.ILogger logger;
        GeoService geoService;

        public Task taskAppSettingsLoop;
        public CancellationToken cancellationTokenAppSettingsLoop;

        public MainViewModel(Serilog.ILogger logger, GeoService geoService, IAppDeviceInfo getDeviceInfo)
        {
            this.logger = logger;
            this.geoService = geoService;

            // Set static properties
            AppVersion = AppInfo.Current.VersionString;
            DeviceID = getDeviceInfo.GetDeviceID();

            // Init properties
            lifeLogEvents = new ObservableCollection<LifeLogEvent>();

            // Start AppSettingsLoop
            taskAppSettingsLoop = Task.Run(() => AppSettingsLoop(), cancellationTokenAppSettingsLoop);

            // Load LifeLog Events
            LoadLifeLogEvents(false).GetAwaiter();
        }

        [ObservableProperty]
        string appVersion;

        [ObservableProperty]
        string deviceID;

        [ObservableProperty]
        string networkAccess;

        [ObservableProperty]
        bool permissionCamera;

        [ObservableProperty]
        bool permissionMicrophone;

        [ObservableProperty]
        bool permissionStorageRead;

        [ObservableProperty]
        bool permissionStorageWrite;

        [ObservableProperty]
        bool permissionLocationWhenInUse;

        [ObservableProperty]
        ObservableCollection<LifeLogEvent> lifeLogEvents;


        [RelayCommand]
        async void RequestPermissions()
        {
            await MainThread.InvokeOnMainThreadAsync(async () => await AppHelper.RequestAllPermissionsAsync());
        }

        [RelayCommand]
        async void Refresh()
        {
            await LoadLifeLogEvents(false);
        }

        [RelayCommand]
        async void ClearAndRefresh()
        {
            await LoadLifeLogEvents(true);
        }

        [RelayCommand]
        void Settings() => AppInfo.Current.ShowSettingsUI();

        [RelayCommand]
        void Tap(Guid id)
        {
            var tapedEvent = LifeLogEvents.First(x => x.Id == id);
            MainThread.BeginInvokeOnMainThread(
                async () =>
                {
                    await Application.Current.MainPage
                        .DisplayAlert("Tap", $"You did tap {tapedEvent.ShortFileName}", "OK");
                });

            //await Shell.Current.GoToAsync($"{nameof(DetailPage)}?Text={s}");
        }

        [RelayCommand]
        async void TakePhoto() => await TakePhotoAsync();

        [RelayCommand]
        async void TakeVideo() => await TakeVideoAsync();

        [RelayCommand]
        void DeleteEventFile(string eventFile)
        {
            if (File.Exists(eventFile))
            {
                File.Delete(eventFile);
            }
            var lifelogEvent = lifeLogEvents.FirstOrDefault(x => x.LocalMediaFileName == eventFile);
            if (lifeLogEvents != null)
            {
                lifeLogEvents.Remove(lifelogEvent);
                SaveLifeLogEventsToSettings();
            }

            Refresh();
        }


        public async void AppSettingsLoop()
        {
            logger.Verbose("{Method}: Start", AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

            while (!cancellationTokenAppSettingsLoop.IsCancellationRequested)
            {
                NetworkAccess = Connectivity.Current.NetworkAccess.ToString();

                PermissionCamera = await Permissions.CheckStatusAsync<Permissions.Camera>() == PermissionStatus.Granted;

                PermissionMicrophone = await Permissions.CheckStatusAsync<Permissions.Microphone>() == PermissionStatus.Granted;

                PermissionStorageRead = await Permissions.CheckStatusAsync<Permissions.StorageRead>() == PermissionStatus.Granted;

                PermissionStorageWrite = await Permissions.CheckStatusAsync<Permissions.StorageWrite>() == PermissionStatus.Granted;

                PermissionLocationWhenInUse = await Permissions.CheckStatusAsync<Permissions.LocationWhenInUse>() == PermissionStatus.Granted;

                await Task.Delay(5000);
            }
        }


        public async Task LoadLifeLogEvents(bool clearAndLoad)
        {
            if (clearAndLoad)
            {
                lifeLogEvents.Clear();

                SaveLifeLogEventsToSettings();
            }

            // Load LifeLogEvents from Settings
            LoadLifeLogEventsFromSettings();

            // Get all local event files
            var files = Directory.GetFiles(AppHelper.AppPathUploadEvents, "*.*", SearchOption.AllDirectories).OrderByDescending(x => new FileInfo(x).CreationTime).ToList();

            // Shorten list
            foreach (var item in lifeLogEvents)
            {
                if (files.Contains(item.LocalMediaFileName))
                {
                    files.Remove(item.LocalMediaFileName);
                }
            }

            // Add new files
            foreach (var f in files)
            {
                var currentLocation = await geoService.GetCurrentLocationAsync();
                var reverseGeoCodeData = await geoService.GetReverseGeocodeDataAsync(currentLocation.Latitude, currentLocation.Longitude);
                var formattedAddress = geoService.BuildFormattedAddress(reverseGeoCodeData.FirstOrDefault());

                var lifelogEvent = new LifeLogEvent()
                {
                    Id = Guid.Parse(Path.GetFileNameWithoutExtension(f)),
                    WhenOrTime = new FileInfo(f).CreationTime.ToUniversalTime(),
                    WhereOrPlace = currentLocation,
                    WhatOrEventOrHowOrAction = formattedAddress,
                    LocalMediaFileName = f,
                };
                lifeLogEvents.Add(lifelogEvent);
            }

            // Save LifeLogEvents to Settings 
            SaveLifeLogEventsToSettings();
        }

        void LoadLifeLogEventsFromSettings()
        {
            var json = Preferences.Get("LifeLogEvents", null);
            if (json == null)
            {
                LifeLogEvents = new ObservableCollection<LifeLogEvent>();
                return;
            }

            LifeLogEvents = Newtonsoft.Json.JsonConvert.DeserializeObject<ObservableCollection<LifeLogEvent>>(json);
        }

        void SaveLifeLogEventsToSettings()
        {
            var json = Newtonsoft.Json.JsonConvert.SerializeObject(lifeLogEvents);

            Preferences.Set("LifeLogEvents", json);
        }


        async Task TakePhotoAsync()
        {
            await MainThread.InvokeOnMainThreadAsync(async () =>
            {
                if (!MediaPicker.Default.IsCaptureSupported)
                {
                    logger.Error("No Camera");
                    await Application.Current.MainPage.DisplayAlert("Error", "No Camera", "Ok");
                    return;
                }

                await Permissions.RequestAsync<Permissions.Camera>();
                await Permissions.RequestAsync<Permissions.StorageRead>();
                await Permissions.RequestAsync<Permissions.StorageWrite>();
                await Permissions.RequestAsync<Permissions.LocationWhenInUse>();

                var photoFile = await MediaHelper.TakePhotoAsync();
                if (photoFile != null)
                {
                    logger.Debug("Photo {File}", photoFile);
                }
                else
                {
                    logger.Error("No Photo was taken");
                    await Application.Current.MainPage.DisplayAlert("Error", "No Photo was taken", "Ok");
                }
            });

            Refresh();
        }

        async Task TakeVideoAsync()
        {
            await MainThread.InvokeOnMainThreadAsync(async () =>
            {
                if (!MediaPicker.Default.IsCaptureSupported)
                {
                    logger.Error("No Camera");
                    await Application.Current.MainPage.DisplayAlert("Error", "No Camera", "Ok");
                }
                else
                {
                    var videoFile = await MediaHelper.TakeVideoAsync();
                    if (videoFile != null)
                    {
                        logger.Debug("Video {File}", videoFile);
                    }
                    else
                    {
                        logger.Error("No Video was taken");
                        await Application.Current.MainPage.DisplayAlert("Error", "No Video was taken", "Ok");
                    }
                }
            });

            Refresh();
        }

    }
}
